@@ -6,7 +6,7 @@
 #
 
 from __future__ import unicode_literals
-import motorcortex
+import motorcortex.motorcortex_pb2
 import logging
 import sys
 import os
@@ -35,7 +35,7 @@ import struct
 motorcortex_parameter_msg = 1
 motorcortex_parameter_list_msg = 2
 
-ParameterMsg = namedtuple('ParameterMsg', 'header, info, value')
+ParameterMsg = namedtuple('ParameterMsg', 'header, info, value, status')
 """Data type which represents parameter description and value"""
 
 ParameterListMsg = namedtuple('ParameterListMsg', 'header, params')
@@ -163,7 +163,7 @@ class MessageTypes(object):
         """
         return self.getNamespace('motorcortex')
 
-    def load(self, proto_hash_pair_list = None):
+    def load(self, proto_hash_pair_list=None):
         """Loads an array of .proto and .json file pairs.
             Args:
                 proto_hash_pair_list([{'hash'-`str`,'proto'-`str`}]): list of hash and proto messages
@@ -340,15 +340,22 @@ class MessageTypes(object):
         if type_inst.decode_value == motorcortex_parameter_msg:
             if msg.status == motorcortex.OK:
                 value_type = self.__types_by_hash[msg.info.data_type]
-                return ParameterMsg(msg.header, msg.info, value_type.decode(msg.value, msg.info.number_of_elements))
+                return ParameterMsg(msg.header, msg.info, value_type.decode(msg.value, msg.info.number_of_elements),
+                                    motorcortex.OK)
             else:
-                return ParameterMsg(msg.header, msg.info, None)
+                return ParameterMsg(msg.header, msg.info, None, msg.status)
         elif type_inst.decode_value == motorcortex_parameter_list_msg:
             params = []
             for param in msg.params:
-                value_type = self.__types_by_hash[param.info.data_type]
-                params.append(ParameterMsg(param.header, param.info,
-                                           value_type.decode(param.value, param.info.number_of_elements)))
+                if msg.status == motorcortex.OK:
+                    value_type = self.__types_by_hash[param.info.data_type]
+                    params.append(ParameterMsg(param.header, param.info,
+                                               value_type.decode(param.value, param.info.number_of_elements,
+                                                                 motorcortex.OK)))
+                else:
+                    params.append(ParameterMsg(param.header, param.info,
+                                               value_type.decode(param.value, param.info.number_of_elements,
+                                                                 msg.status)))
             return ParameterListMsg(msg.header, params)
 
         return msg
