@@ -16,6 +16,16 @@ import os
 import queue
 from concurrent.futures import ThreadPoolExecutor
 from pynng import Req0, TLSConfig
+from enum import Enum
+
+
+class ConnectionState(Enum):
+    CONNECTING = 0
+    CONNECTION_OK = 1
+    CONNECTION_LOST = 2
+    CONNECTION_FAILED = 3
+    DISCONNECTING = 4
+    DISCONNECTED = 5
 
 
 class Base64Encoder(json.JSONEncoder):
@@ -47,12 +57,15 @@ class Request(object):
         self.__protobuf_types = protobuf_types
         self.__parameter_tree = parameter_tree
         self.__pool = ThreadPoolExecutor(max_workers=1)
+        self.__clb = ThreadPoolExecutor(max_workers=1)
+        self.__connection_state = ConnectionState.DISCONNECTED
 
     def url(self):
         return self.__url
 
     def connect(self, url, **kwargs):
-        conn_timeout_ms, recv_timeout_ms, certificate = self.parse(**kwargs)
+        self.__connection_state = ConnectionState.CONNECTING
+        conn_timeout_ms, recv_timeout_ms, certificate, state_update = self.parse(**kwargs)
         self.__url = url
         tls_config = None
         if certificate:
@@ -63,11 +76,22 @@ class Request(object):
 
         def pre_connect_cb(pipe):
             self.__connected_lock.put(True)
+            self.__connection_state = ConnectionState.CONNECTION_OK
+            if state_update:
+                self.__clb.submit(state_update, self, self.connectionState())
 
         def post_remove_cb(pipe):
+            if self.__connection_state == ConnectionState.DISCONNECTING:
+                self.__connection_state = ConnectionState.DISCONNECTED
+            elif self.__connection_state == ConnectionState.CONNECTING:
+                self.__connection_state = ConnectionState.CONNECTION_FAILED
+            elif self.__connection_state == ConnectionState.CONNECTION_OK:
+                self.__connection_state = ConnectionState.CONNECTION_LOST
             self.__connected_lock.put(False)
+            if state_update:
+                self.__clb.submit(state_update, self, self.connectionState())
 
-        self.__socket.add_pre_pipe_connect_cb(pre_connect_cb)
+        self.__socket.add_post_pipe_connect_cb(pre_connect_cb)
         self.__socket.add_post_pipe_remove_cb(post_remove_cb)
         self.__socket.dial(url)
 
@@ -75,6 +99,7 @@ class Request(object):
                                         conn_timeout_ms / 1000.0))
 
     def close(self):
+        self.__connection_state = ConnectionState.DISCONNECTING
         if self.__connected_lock:
             self.__connected_lock.put(False)
         self.__socket.close()
@@ -110,6 +135,9 @@ class Request(object):
 
         return self.send(self.__protobuf_types.encode(login_msg))
 
+    def connectionState(self):
+        return self.__connection_state
+
     def getParameterTreeHash(self):
         """Request a parameter tree hash from the server.
 
@@ -401,11 +429,12 @@ class Request(object):
         return release_param_msg
 
     @staticmethod
-    def parse(conn_timeout_ms=0, timeout_ms=None, recv_timeout_ms=None, certificate=None, login=None, password=None):
+    def parse(conn_timeout_ms=0, timeout_ms=None, recv_timeout_ms=None, certificate=None, login=None, password=None,
+              state_update=None):
         if timeout_ms and not conn_timeout_ms:
             conn_timeout_ms = timeout_ms
 
-        return conn_timeout_ms, recv_timeout_ms, certificate
+        return conn_timeout_ms, recv_timeout_ms, certificate, state_update
 
     @staticmethod
     def __send(req, encoded_msg, protobuf_types):
